/**
 * The GEsture Clustering ToolKit (GECKo) (.NET Framework 4.0 C# version)
 *
 * 	    Radu-Daniel Vatavu, Ph.D.
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 *	    Lisa Anthony, Ph.D.
 *      UMBC
 *      Information Systems Department
 *      1000 Hilltop Circle
 *      Baltimore, MD 21250
 *      lanthony@umbc.edu
 *
 *	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publication for GECKo, and what should be 
 * used to cite it, is:
 *
 *	Anthony, L., Vatavu, R.-D., and Wobbrock, J.O. (2013).  
 *	  Understanding the Consistency of Users' Pen and Finger Stroke Gesture Articulation. 
 *	  Proceedings of the 39th Graphics Interface Conference - GI'13 
 *	  (Regina, Saskatchewan, Canada, May 2013). Toronto, Ontario: Canadian Information Processing Society.
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2013, Radu-Daniel Vatavu, Lisa Anthony, and 
 * Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, 
 *	    University of Washington, nor UMBC, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Lisa Anthony
 * OR Jacob O. Wobbrock BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/
using System;
using System.Collections.Generic;

namespace GestureToolkit
{
    public delegate float GestureMetric(Gesture g1, Gesture g2);

    public class GestureMetrics
    {
        /// <summary>
        /// Euclidean distance
        /// Complexity: O(n)
        /// </summary>
        public static float Euclidean(Gesture g1, Gesture g2)
        {
            Point[] points1 = null, points2 = null;
            int n = GetSampledPoints(g1, g2, ref points1, ref points2);

            float d = 0;
            for (int i = 0; i < n; i++)
                d += Measurements.SqrEuclideanDistance(points1[i], points2[i]);

            return d;
        }

        private static int GetSampledPoints(
            Gesture g1, Gesture g2,
            ref Point[] points1, ref Point[] points2)
        {
            points1 = g1.SampledPoints;
            points2 = g2.SampledPoints;

            if (points1.Length != points2.Length)
                throw new InvalidOperationException(
                    String.Format(
                        "Metric requirement not met: gestures are not sampled equally ({0} vs {1} points)",
                        points1.Length,
                        points2.Length
                    ));

            return points1.Length;
        }
    }
}

